Attribute VB_Name = "MFileSystem"
' Filename routines
' 12.23.00
'
' getExtension      returns extension of a file
' getFile           extracts filename from a full path
' getPath           extracts path from a full path
' joinFilePath      returns full path from a filename and path

Option Explicit

Private Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindNextFile Lib "kernel32" Alias "FindNextFileA" (ByVal hFindFile As Long, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindClose Lib "kernel32" (ByVal hFindFile As Long) As Long
Private Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" (ByVal hWnd As Long, ByVal lpOperation As String, ByVal lpFile As String, ByVal lpParameters As String, ByVal lpDirectory As String, ByVal nShowCmd As Long) As Long

Private Const MAX_PATH = 260
Private Const MAXDWORD = &HFFFF
Private Const INVALID_HANDLE_VALUE = -1

Private Const FILE_ATTRIBUTE_ARCHIVE = &H20
Private Const FILE_ATTRIBUTE_COMPRESSED = &H800
Private Const FILE_ATTRIBUTE_DIRECTORY = &H10
Private Const FILE_ATTRIBUTE_HIDDEN = &H2
Private Const FILE_ATTRIBUTE_NORMAL = &H80
Private Const FILE_ATTRIBUTE_READONLY = &H1
Private Const FILE_ATTRIBUTE_SYSTEM = &H4
Private Const FILE_ATTRIBUTE_TEMPORARY = &H100

Private Type FILETIME
        dwLowDateTime As Long
        dwHighDateTime As Long
End Type

Private Type WIN32_FIND_DATA
        dwFileAttributes As Long
        ftCreationTime As FILETIME
        ftLastAccessTime As FILETIME
        ftLastWriteTime As FILETIME
        nFileSizeHigh As Long
        nFileSizeLow As Long
        dwReserved0 As Long
        dwReserved1 As Long
        cFileName As String * MAX_PATH
        cAlternate As String * 14
End Type
Public Function OpenFile(sFilename As String, Optional sCommand As String = "open") As Long
OpenFile = ShellExecute(0, sCommand, sFilename, "", "", 0)
End Function
Public Function getExtension(ByVal File As String) As String
Dim iLoc As Integer
File = getFile(File)
iLoc = InStr(StrReverse(File), ".")
If iLoc > 0 Then
    getExtension = Mid$(File, Len(File) - iLoc + 2)
Else
    getExtension = ""
End If
End Function
Public Function FileList(Path As String, Optional FileAttribute As VbFileAttribute = vbNormal) As String()
Dim asList() As String
Dim sFile As String

sFile = Dir(Path, FileAttribute)
Do Until sFile = ""
    
    'check for null array
    On Error Resume Next
    ReDim Preserve asList(UBound(asList) + 1)
    If Err Then ReDim asList(0)
    On Error GoTo 0
    
    asList(UBound(asList)) = sFile
    sFile = Dir()
Loop
FileList = asList
End Function
Public Function DirExists(d As String) As Boolean
DirExists = (Dir(d, vbDirectory) <> "")
End Function

Public Function FileExists(f As String) As Boolean
FileExists = (Dir(f, vbArchive Or vbHidden Or vbNormal Or vbReadOnly Or vbSystem) <> "")
End Function
Public Function getFile(ByVal File As String, Optional ByVal NoExtension As Boolean = False) As String
Dim iLoc As Integer

File = Replace(File, "/", "\")

iLoc = InStr(StrReverse(File), "\")
If iLoc > 0 Then File = Mid$(File, Len(File) - iLoc + 2)

If NoExtension Then
    iLoc = InStr(File, ".")
    If iLoc > 0 Then File = Left(File, iLoc - 1)
End If

getFile = File

End Function
Public Function getPath(File As String) As String
Dim iLoc As Integer
iLoc = InStr(StrReverse(File), "\")
If iLoc > 0 Then
    getPath = Left$(File, Len(File) - iLoc + 1)
Else
    getPath = ""
End If
End Function
Public Function joinFilePath(Path As String, File As String)
'faster than IIF(...)
If Right(Path, 1) <> "\" Then
    joinFilePath = Path & "\" & File
Else
    joinFilePath = Path & File
End If
End Function
Public Function KillTree(Path As String) As Boolean
Dim cDirs As New Collection, cFiles As New Collection
cDirs.Add Path

Dim hFindFile As Long, ffd As WIN32_FIND_DATA
Dim lRetVal As Long, sCurDir As String, sContents As String, sFilename As String
Dim iCount As Long

Do While cDirs.Count > 0
    sCurDir = cDirs.Item(cDirs.Count)
    
    sContents = joinFilePath(sCurDir, "*.*")
    
    If FileExists(sContents) Then Kill sContents
    
    sFilename = Dir(sContents, vbDirectory)
    iCount = 0
    Do Until sFilename = ""
        If sFilename <> "." And sFilename <> ".." Then
            iCount = iCount + 1
            cDirs.Add joinFilePath(sCurDir, sFilename)
        End If
        sFilename = Dir()
    Loop
    
    If iCount = 0 Then
        'no contents, attempt to kill dir
        'On Error Resume Next
        RmDir sCurDir & "\"
        If Not Err Then cDirs.Remove cDirs.Count
        'On Error GoTo 0
    End If
    
'    hFindFile = FindFirstFile(sFile, ffd)
'    If hFindFile = INVALID_HANDLE_VALUE Then
'        'no contents, remove directory, remove from collection
'        lRetVal = 0
'    Else
'        lRetVal = hFindFile
'    End If
    
'    Do Until lRetVal = 0
'
'        'remove null characters
'        sFilename = ffd.cFileName
'        Dim iLoc As Integer
'        iLoc = InStr(1, sFilename, vbNullChar, 0)
'        If iLoc <> 0 Then sFilename = Left(sFilename, iLoc - 1)
'
'        If sFilename <> "." And sFilename <> ".." Then
'            If ffd.dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY Then
'                'dir, add to buffer
'                cDirs.Add joinFilePath(sFilename, sCurDir)
'            End If
'        End If
'        lRetVal = FindNextFile(hFindFile, ffd)
'    Loop
'    FindClose hFindFile
    
Loop
End Function
